import { Mapper } from '@automapper/core';
import { InjectMapper } from '@automapper/nestjs';
import { CommandHandler } from '@nestjs/cqrs';
import { LoggingMessager } from '../../adapters/secondaries/logging.messager';
import { TerritoriesRepository } from '../../adapters/secondaries/territories.repository';
import { TerritoryMessager } from '../../adapters/secondaries/territory.messager';
import { UpdateTerritoryCommand } from '../../commands/update-territory.command';
import { UpdateTerritoryRequest } from '../dtos/update-territory.request';
import { Territory } from '../entities/territory';

@CommandHandler(UpdateTerritoryCommand)
export class UpdateTerritoryUseCase {
  constructor(
    private readonly _repository: TerritoriesRepository,
    private readonly _territoryMessager: TerritoryMessager,
    private readonly _loggingMessager: LoggingMessager,
    @InjectMapper() private readonly _mapper: Mapper,
  ) {}

  async execute(command: UpdateTerritoryCommand): Promise<Territory> {
    const entity = this._mapper.map(
      command.updateTerritoryRequest,
      UpdateTerritoryRequest,
      Territory,
    );

    try {
      const territory = await this._repository.update(
        command.updateTerritoryRequest.uuid,
        entity,
      );
      this._territoryMessager.publish(
        'update',
        JSON.stringify(command.updateTerritoryRequest),
      );
      this._loggingMessager.publish(
        'territory.update.info',
        JSON.stringify(command.updateTerritoryRequest),
      );
      return territory;
    } catch (error) {
      this._loggingMessager.publish(
        'territory.update.crit',
        JSON.stringify({
          command,
          error,
        }),
      );
      throw error;
    }
  }
}
